/* ooLiftSpec -
 * This will make ordered.lft and random.lft 'lift specifications' in
 * each chromosome subdir of ooDir based on cloneMap. */
#include "common.h"
#include "linefile.h"
#include "portable.h"
#include "hash.h"

void usage()
/* Explain usage and exit. */
{
errAbort(
 "ooLiftSpec - make a specification for lifting coordinates from\n"
 "contig to chromosome\n"
 "usage:\n"
 "   ooLiftSpec cloneMap ooDir\n"
 "This will make lift directory containing ordered.lft and random.lft\n"
 "each chromosome subdir of ooDir based on cloneMap.\n");
}

struct contigInfo 
/* Info on one contig. */
    {
    struct contigInfo *next;	/* Next in list */
    char *name;			/* Name of contig. */
    int size;			/* Size of contig. */
    };

void writeListType(char *fileName, struct contigInfo *ciList, char *suffix)
/* Write file list... */
{
FILE *f = mustOpen(fileName, "w");
struct contigInfo *ci;

for (ci = ciList; ci != NULL; ci = ci->next)
    fprintf(f, "%s/%s%s\n", ci->name, ci->name, suffix);
fclose(f);
}

void writeList(char *fileName, struct contigInfo *ciList)
/* Write list file that contains all contigs. */
{
FILE *f = mustOpen(fileName, "w");
struct contigInfo *ci;

for (ci = ciList; ci != NULL; ci = ci->next)
    fprintf(f, "%s\n", ci->name);
fclose(f);
}

enum {spacing=200000};

void writeLift(char *fileName, struct contigInfo *ciList, char *chromName, char *shortName)
/* Write list file that contains all contigs. */
{
FILE *f = mustOpen(fileName, "w");
struct contigInfo *ci, *prev = NULL;
int offset = 0;
int size = 0;

for (ci = ciList; ci != NULL; ci = ci->next)
    {
    if (prev != NULL)
    	size += spacing;
    size += ci->size;
    prev = ci;
    }
uglyf("%s %s size %d\n", fileName, chromName, size);
for (ci = ciList; ci != NULL; ci = ci->next)
    {
    fprintf(f, "%d\t%s/%s\t%d\t%s\t%d\n", offset, shortName, ci->name, ci->size, chromName, size);
    offset += ci->size;
    offset += spacing;
    }
fclose(f);
}

void writeCtgPos(char *fileName, struct contigInfo *ciList, char *chromName)
/* Write contig position table for database. */
{
struct contigInfo *ci, *prev = NULL;
int offset = 0;
FILE *f = mustOpen(fileName, "a");

for (ci = ciList; ci != NULL; ci = ci->next)
    {
    fprintf(f, "%s\t%d\t%s\t%d\t%d\n", ci->name, ci->size, chromName, offset, 
    	offset+ci->size);
    offset += ci->size + spacing;
    }
fclose(f);
}

int sizeFromAgp(char *fileName)
/* Return length of sequence from .agp file. */
{
char prevLine[512];
struct lineFile *lf;
int lineSize, wordCount;
char *line, *words[16];
boolean gotAny = FALSE;
int size;

if (!fileExists(fileName))
    return 0;
lf = lineFileOpen(fileName, TRUE);
while (lineFileNext(lf, &line, &lineSize))
    {
    gotAny = TRUE;
    if (lineSize >= sizeof(prevLine))
	errAbort("Line too long line %d of %s\n", lf->lineIx, lf->fileName);
    memcpy(prevLine, line, lineSize+1);
    }
if (!gotAny)
    return 0;
wordCount = chopLine(line, words);
if (wordCount < 8)
    errAbort("Expecting at least 8 words on last line of %s\n", lf->fileName);
size = atoi(words[2]);
lineFileClose(&lf);
return size;
}

void fillInSizes(struct contigInfo *ciList, char *ooDir, char *chrom)
/* Fill in size of contigs from .agp files. */
{
char fileName[512];
struct contigInfo *ci;

for (ci = ciList; ci != NULL; ci = ci->next)
    {
    sprintf(fileName, "%s/%s/%s/%s.agp", ooDir, chrom, ci->name, ci->name);
    ci->size = sizeFromAgp(fileName);
    }
}

struct contigInfo *readContigs(struct lineFile *lf)
/* Read contigs up intil next "start human" or "end human" */
{
int lineSize, wordCount;
char *line, *words[16];
struct contigInfo *ciList = NULL, *ci;
struct hash *hash = newHash(0);
char *contig;

while (lineFileNext(lf, &line, &lineSize))
    {
    if (startsWith("end human", line))
	break;
    if (startsWith("start human", line))
	{
	lineFileReuse(lf);
	break;
	}
    if (line[0] == '@' || line[0] == '*')
	continue;
    wordCount = chopLine(line, words);
    if (wordCount < 6)
	errAbort("short line %d of %s\n", lf->lineIx, lf->fileName);
    contig = words[3];
    if (!hashLookup(hash, contig))
	{
	hashAdd(hash, contig, NULL);
	AllocVar(ci);
	ci->name = cloneString(contig);
	slAddHead(&ciList, ci);
	}
    }
freeHash(&hash);
slReverse(&ciList);
return ciList;
}

void ooLiftSpec(char *mapName, char *ooDir)
/* This will make ordered.lft and random.lft 'lift specifications' in
 * each chromosome subdir of ooDir based on cloneMap. */
{
struct lineFile *lf = lineFileOpen(mapName, TRUE);
int lineSize, wordCount;
char *line, *words[16];
char chromName[32];
char shortName[32];
boolean isOrdered;
char *chrom;
struct contigInfo *ciList, *ci;
char fileName[512];
char *shortType, *type, *suffix;
char liftDir[512];

while (lineFileNext(lf, &line, &lineSize))
    {
    if (!startsWith("start human SUPERLINK", line))
	continue;
    wordCount = chopLine(line, words);
    if (wordCount != 5 && wordCount != 4)
	errAbort("Odd start line %d of %s\n", lf->lineIx, lf->fileName);
    if (words[wordCount-1][0] != '*')
	errAbort("Odd start line %d of %s\n", lf->lineIx, lf->fileName);
    chrom = strrchr(words[2], '.');
    if (chrom == NULL)
	errAbort("Couldn't find chromosome line %d of 5s\n", lf->lineIx, lf->fileName);
    chrom += 1;
    if (sameString(chrom, "COMMIT"))
	continue;
    isOrdered = sameWord(words[3], "ORDERED");
    if (isOrdered)
	{
	type = "ordered";
	shortType = "o";
	suffix = "";
	}
    else
	{
	type = "random";
	shortType = "r";
	suffix = "_random";
	}
    sprintf(chromName, "chr%s%s", chrom, suffix);
    printf("%s %s\n", chrom, chromName);
    strcpy(shortName, chrom);
    if (sameString(chrom, "NA"))
	{
	AllocVar(ciList);
	ciList->name = "ctg_na";
	}
    else
	ciList = readContigs(lf);
    fillInSizes(ciList, ooDir, shortName);

    sprintf(liftDir, "%s/%s/lift", ooDir, shortName);
    mkdir(liftDir, 0777);
    sprintf(fileName, "%s/%s.lst", liftDir, type);
    writeList(fileName, ciList);
    sprintf(fileName, "%s/%s.lft", liftDir, type);
    writeLift(fileName, ciList, chromName, shortName);
    sprintf(fileName, "%s/%sOut.lst", liftDir, shortType);
    writeListType(fileName, ciList, ".fa.out");
    sprintf(fileName, "%s/%sAgp.lst", liftDir, shortType);
    writeListType(fileName, ciList, ".agp");
    sprintf(fileName, "%s/ctgPos.tab", liftDir);
    writeCtgPos(fileName, ciList, chromName);
    }
lineFileClose(&lf);
}


int main(int argc, char *argv[])
/* Process command line. */
{
if (argc != 3)
    usage();
ooLiftSpec(argv[1], argv[2]);
}
